#!/usr/bin/env python2
# -*- coding: utf-8 -*-

import sys, random, pygame
from pygame.locals import *

# Constantes du jeu
UP,LEFT,DOWN,RIGHT = 0,1,2,3

SOL,MUR,COEUR = "s","m","c"
TETE,CORPS,QUEUE = ["th","tg","tb","td"],["ch","cg","cb","cd"],["qh","qg","qb","qd"]

lc,hc = 80, 60          # Hauteur/largeur (cases)
tc = 10                 # Taille d'une case (pixels)
lf,hf = lc * tc,hc * tc # Hauteur/largeur (pixels)

# Pixel art
sol   = pygame.image.load("sand.png")
mur   = pygame.image.load("wall.png")
coeur = pygame.image.load("star.png")
tete  = pygame.image.load("head.png")
corps = pygame.image.load("body.png")
queue = pygame.image.load("tail.png")

# Fonctions utilitaires :
# > Primitive pour le calcul du mouvement
def pas (x,y,d,n):
	if d == UP:
		return (x,y-n)
	elif d == DOWN:
		return (x,y+n)
	elif d == LEFT:
		return (x-n,y)
	elif d == RIGHT:
		return (x+n,y)

# > Positions suivante/précédente par rapport à d
def avant (x,y,d):
	return pas(x,y,d,1)
def arriere (x,y,d):
	return pas(x,y,d,-1)

# > Direction d'un objet quelconque
def direc(type_o, objet):
	return type_o.index(objet)

# > Retourne l'ancle d'une diréction par rapport au haut
def angle(d):
	return 90*d

# > Retourne le rectangle correspondant à une case
def case(x,y):
	return (x*tc,y*tc,tc,tc)

def img(objet):
	if objet == SOL:
		return sol
	if objet == MUR:
		return mur
	if objet == COEUR:
		return coeur
	if objet in CORPS:
		return pygame.transform.rotate(corps,angle(direc(CORPS,objet)))
	if objet in TETE:
		return pygame.transform.rotate(tete,angle(direc(TETE,objet)))
	if objet in QUEUE:
		return pygame.transform.rotate(queue,angle(direc(QUEUE,objet)))

# Carte du niveau
# Valeurs implicites :
#  sol <-> blanc
#  mur <-> rouge
#  tete <-> vert (le niveau de bleu donne la direction)

carte = pygame.image.load("level0.png")

# Retourne la valeur implicite d'une couleur
def valeur_carte(couleur):
	if couleur == (255,255,255):
		return SOL
	if couleur == (255,0,0):
		return MUR
	for d in [UP,DOWN,LEFT,RIGHT]:
		if couleur == (0,255,d):
			return TETE[d]
	sys.stderr.write ("ERREUR : Couleur inattendue sur la carte : ",c,"\n")
	exit()

# Les positions de la tête et de la queue sont mémoïsées
ptx,pty = -1,-1 # Position de la tête
pqx,pqy = -1,-1 # Position de la queue

# Génération de la carte en mémoire
niveau = [[]]*lc
for i in range(lc):
	niveau[i] = [""]*hc
	for j in range(hc):
		niveau[i][j] = valeur_carte(carte.get_at((i, j)))
		if niveau[i][j] in TETE:
			ptx,pty = i,j

# Placement de la queue derrière la tête
(pqx,pqy) = arriere(ptx, pty, direc(TETE, niveau[ptx][pty]))
niveau[pqx][pqy] = QUEUE[direc(TETE, niveau[ptx][pty])]

# Placement d'un cœur au sol au hasard
dpsx,dpsy = ptx,pty
while (dpsx == ptx and dpsy == pty) or niveau[dpsx][dpsy] != SOL:
	# Peut se bloquer. TODO: Vérifier qu'il y a bien du sol.
	dpsx,dpsy = random.choice(range(lc)),random.choice(range(hc))
niveau[dpsx][dpsy] = COEUR

if ptx < 0 or pqx < 0:
	sys.stderr.write ("ERREUR : Tête/queue non trouvée sur la carte.\n")
	exit()

# Trucs importants
pygame.init()
fenetre = pygame.display.set_mode((lf,hf))
ips = pygame.time.Clock()

vitesse = 10    # Vitesse du jeu (en tics/s).
croissance = 10 # Croissance restant à effectuer

# Boucle principale
while True:
	# Vérification des évènements
	for event in pygame.event.get():
		if event.type == KEYDOWN:
			if event.key == K_ESCAPE:
				print ("Touche ECHAP appuyée ; fin du jeu.")
				exit()
			elif event.key == K_UP:
				niveau[ptx][pty] = TETE[UP]
			elif event.key == K_DOWN:
				niveau[ptx][pty] = TETE[DOWN]
			elif event.key == K_LEFT:
				niveau[ptx][pty] = TETE[LEFT]
			elif event.key == K_RIGHT:
				niveau[ptx][pty] = TETE[RIGHT]
	
	# Mouvement de la tête
	d = direc(TETE, niveau[ptx][pty])
	(ptx2,pty2) = avant(ptx,pty,d)
	
	if niveau[ptx2][pty2] == MUR:
		print ("Mur percuté ! Game over.")
		exit()
	
	if niveau[ptx2][pty2] in CORPS or niveau[ptx2][pty2] in QUEUE:
		print ("Ouroboros ! Game over.")
		exit()

	if niveau[ptx2][pty2] == COEUR:
		croissance += 3
		dpsx,dpsy = ptx2,pty2
		while (dpsx == ptx2 and dpsy == pty2) or niveau[dpsx][dpsy] != SOL:
			# Peut se bloquer. TODO: Vérifier qu'il y a bien du sol.
			dpsx,dpsy = random.choice(range(lc)),random.choice(range(hc))
		niveau[dpsx][dpsy] = COEUR
	
	niveau[ptx2][pty2] = TETE[d]
	niveau[ptx][pty] = CORPS[d]
	ptx,pty = ptx2,pty2
	
	# Mouvement de la queue
	if croissance == 0:
		d = direc(QUEUE,niveau[pqx][pqy])
		(pqx2,pqy2) = avant(pqx,pqy,d)
		if not niveau[pqx2][pqy2] in CORPS:
			sys.stderr.write("ERREUR : Serpent incohérent.")
			exit()
		niveau[pqx2][pqy2] = QUEUE[direc(CORPS,niveau[pqx2][pqy2])]
		niveau[pqx][pqy] = SOL
		pqx,pqy = pqx2,pqy2
	else:
		croissance -= 1
	
	# Dessin du niveau
	for i in range(lc):
		for j in range(hc):
			fenetre.blit(img(niveau[i][j]),case(i,j))
	
	pygame.display.update()
	ips.tick(vitesse)
